@@ -1,3 +1,4 @@
+use std::cmp::Ordering;
 use std::collections::VecDeque;
 
 use serde::{de, forward_to_deserialize_any};
@@ -6,7 +7,7 @@ use super::{
     seq::{ItemKind, PairSeq},
     Pair, Stash,
 };
-use crate::de::Deserializer;
+use crate::de::Value;
 use crate::error::{Error, Result};
 
 pub(crate) struct PairMap<'de> {
@@ -15,6 +16,49 @@ pub(crate) struct PairMap<'de> {
     stash: Stash<'de>,
 }
 
+#[derive(Copy, Clone)]
+enum KeyType<'a> {
+    Number(u16),
+    Name(&'a [u8]),
+    None,
+}
+
+impl<'a> PartialOrd for KeyType<'a> {
+    fn partial_cmp(&self, other: &KeyType) -> Option<Ordering> {
+        if let KeyType::Number(num1) = self {
+            if let KeyType::Number(num2) = other {
+                Some(num1.cmp(num2))
+            } else {
+                Some(Ordering::Greater)
+            }
+        } else if let KeyType::Number(_) = other {
+            Some(Ordering::Less)
+        } else {
+            None
+        }
+    }
+}
+
+impl<'a> PartialEq for KeyType<'a> {
+    fn eq(&self, other: &KeyType<'a>) -> bool {
+        if let KeyType::Number(num1) = self {
+            if let KeyType::Number(num2) = other {
+                num1 == num2
+            } else {
+                false
+            }
+        } else if let KeyType::Name(name1) = self {
+            if let KeyType::Name(name2) = other {
+                name1 == name2
+            } else {
+                false
+            }
+        } else {
+            false
+        }
+    }
+}
+
 impl<'de> PairMap<'de> {
     pub(crate) fn new(depth: u16, pairs: VecDeque<Pair<'de>>) -> Self {
         Self {
@@ -115,30 +159,30 @@ impl<'de> PairMap<'de> {
     }
 
     pub(crate) fn into_seq(self) -> Result<PairSeq<'de>> {
-        let remaining_depth = self.stash.remaining_depth;
-
         let pairs = self.into_pairs()?;
         let mut items = vec![];
-        let mut sorted_items = vec![];
 
         for (key, value) in pairs {
-            if let Ok(index) = crate::from_bytes::<u16>(key) {
-                sorted_items.push((index as isize, value));
+            if key.is_empty() {
+                items.push((KeyType::None, value))
             } else {
-                items.push((-1, value));
+                let index: Result<u16> = serde::de::Deserialize::deserialize(&mut Value::new(key));
+                if let Ok(index) = index {
+                    items.push((KeyType::Number(index), value));
+                } else {
+                    items.push((KeyType::Name(key), value));
+                }
             }
         }
 
         // Order the items by their keys
-        // TODO: this is overcomplicated, look for an easier way
-        sorted_items.sort_by_key(|item| item.0);
-        sorted_items.dedup_by_key(|item| item.0);
-        items.append(&mut sorted_items);
+        items.sort_by(|a, b| a.0.partial_cmp(&b.0).unwrap_or(Ordering::Equal));
         items.reverse();
+        items.dedup_by(|a, b| a.0 == b.0);
 
         let items = items.into_iter().map(|item| item.1).collect();
 
-        Ok(PairSeq::new(items, remaining_depth))
+        Ok(PairSeq::new(items))
     }
 }
 
@@ -206,14 +250,14 @@ impl<'de> de::MapAccess<'de> for PairMap<'de> {
         }
 
         if let Some(key) = self.next_key()? {
-            return seed.deserialize(&mut Deserializer::new(key)).map(Some);
+            return seed.deserialize(&mut Value::new(key)).map(Some);
         }
 
         // Visit stash
         let key = self.stash.next_key()?;
 
         match key {
-            Some(key) => seed.deserialize(&mut Deserializer::new(&key)).map(Some),
+            Some(key) => seed.deserialize(&mut Value::new(key)).map(Some),
             None => Ok(None),
         }
     }
@@ -223,10 +267,7 @@ impl<'de> de::MapAccess<'de> for PairMap<'de> {
         V: de::DeserializeSeed<'de>,
     {
         match self.next_value() {
-            Ok(value) => seed.deserialize(&mut Deserializer::new_with_depth(
-                value,
-                self.stash.remaining_depth - 1,
-            )),
+            Ok(value) => seed.deserialize(&mut Value::new(value)),
             _ => seed.deserialize(self.stash.next_value_map()?),
         }
     }
@@ -245,14 +286,21 @@ impl<'de> de::EnumAccess<'de> for &mut PairMap<'de> {
             return Err(Error::MaximumDepthReached);
         }
 
-        let key = {
-            if let Some(key) = self.next_key()? {
-                key
-            } else {
-                // Visit stash
-                let key = self.stash.next_key()?;
+        // We throw all keys away, except the last one
+        let mut last_key = None;
+        while let Some(key) = self.next_key()? {
+            last_key = Some(key);
+        }
 
-                match key {
+        let key = match last_key {
+            Some(key) => key,
+            None => {
+                // Visit stash
+                while let Some(key) = self.stash.next_key()? {
+                    last_key = Some(key)
+                }
+
+                match last_key {
                     Some(key) => key,
                     None => {
                         return Err(Error::EofReached);
@@ -261,13 +309,7 @@ impl<'de> de::EnumAccess<'de> for &mut PairMap<'de> {
             }
         };
 
-        Ok((
-            seed.deserialize(&mut Deserializer::new_with_depth(
-                key,
-                self.stash.remaining_depth - 1,
-            ))?,
-            self,
-        ))
+        Ok((seed.deserialize(&mut Value::new(key))?, self))
     }
 }
 
@@ -284,10 +326,7 @@ impl<'de> de::VariantAccess<'de> for &mut PairMap<'de> {
         V: de::Visitor<'de>,
     {
         match self.next_value() {
-            Ok(value) => {
-                let mut de = Deserializer::new_with_depth(value, self.stash.remaining_depth - 1);
-                serde::de::Deserializer::deserialize_seq(&mut de, visitor)
-            }
+            Ok(value) => serde::de::Deserializer::deserialize_seq(&mut Value::new(value), visitor),
             _ => visitor.visit_seq(&mut self.stash.next_value_map()?.into_seq()?),
         }
     }
@@ -304,10 +343,7 @@ impl<'de> de::VariantAccess<'de> for &mut PairMap<'de> {
         T: de::DeserializeSeed<'de>,
     {
         match self.next_value() {
-            Ok(value) => seed.deserialize(&mut Deserializer::new_with_depth(
-                value,
-                self.stash.remaining_depth - 1,
-            )),
+            Ok(value) => seed.deserialize(&mut Value::new(value)),
             _ => seed.deserialize(self.stash.next_value_map()?),
         }
     }
